@@ -26,9 +26,15 @@ module ActiveRecord
 
     private
     def hash_rows
-      @hash_rows ||= @rows.map { |row|
-        Hash[@columns.zip(row)]
-      }
+      @hash_rows ||=
+        begin
+          # We freeze the strings to prevent them getting duped when
+          # used as keys in ActiveRecord::Model's @attributes hash
+          columns = @columns.map { |c| c.dup.freeze }
+          @rows.map { |row|
+            Hash[columns.zip(row)]
+          }
+        end
     end
   end
 end
